import { Component, Element, Listen, Method, Prop, State, Event, h, Watch, Host } from '@stencil/core';
import { unifyEvent, isMobile, isFullscreen, debounce, isAndroidTablet, isIPad } from '@deckdeckgo/utils';
import { getSlideDefinition, getAttributesDefinition } from '@deckdeckgo/deck-utils';
import { DeckdeckgoDeckBackgroundUtils } from '../utils/background-utils';
import { HideSlides, RevealSlide, AnimationSlide } from '../utils/animation';
import { containerSize } from '../utils/size.utils';
export class DeckGoDeck {
  constructor() {
    this.keyboard = true;
    this.embedded = false;
    this.cloneBackground = true;
    this.rtl = false;
    this.startX = null;
    this.startY = null;
    this.deckMove = 0;
    this.autoSwipeRatio = 10;
    this.block = false;
    this.activeIndex = 0;
    this.length = 0;
    this.fullscreen = false;
    this.cursorHidden = false;
    this.idleMouseTimeout = 2000;
    this.reveal = true;
    this.revealOnMobile = false;
    this.animation = 'slide';
    this.direction = 'horizontal';
    this.directionMobile = 'papyrus';
    this.autoSlide = 'false';
    this.autoSlideInterval = 5000;
    // We do not consider iPad and Tablet as "mobile" devices. With mobile we mean smaller devices, phones.
    this.mobile = isMobile() && !(isIPad() || isAndroidTablet());
    this.debounceWindowResize = debounce(async () => {
      await this.initSlideSize();
      if (this.dir !== 'papyrus') {
        await this.slideTo(this.activeIndex);
      }
      const toggleFullscreen = isFullscreen();
      await this.hideOrClearMouseCursorTimer(toggleFullscreen);
      await this.showHideActionsSlot(toggleFullscreen);
    }, 100);
    this.onIntersection = async (entries) => {
      if (!entries || entries.length <= 0) {
        return;
      }
      const intersectingElements = entries
        .filter((entry) => entry.isIntersecting)
        .map((entry) => entry.target);
      if (!intersectingElements || intersectingElements.length <= 0) {
        return;
      }
      const promises = intersectingElements.map((element) => {
        return new Promise(async (resolve) => {
          this.observer.unobserve(element);
          await element.lazyLoadContent();
          resolve();
        });
      });
      await Promise.all(promises);
    };
    this.keyboardAssist = async ($event) => {
      if ($event.defaultPrevented) {
        return;
      }
      if (['ArrowLeft', 'k', 'PageUp'].indexOf($event.key) !== -1) {
        await this.slideNextPrev(false, true, true);
      }
      else if (['ArrowRight', 'j', 'PageDown'].indexOf($event.key) !== -1) {
        await this.slideNextPrev(true, true, true);
      }
    };
  }
  async componentWillLoad() {
    await this.initRtl();
    await this.initDirection();
  }
  async componentDidLoad() {
    await this.initSlideSize();
    await this.initAutoSlide();
    this.initWindowResize();
    this.initKeyboardAssist();
  }
  disconnectedCallback() {
    if (this.observer) {
      this.observer.disconnect();
    }
    if (this.idleSlideLoopTimer > 0) {
      clearTimeout(this.idleSlideLoopTimer);
    }
    if (this.slideLoopInterval > 0) {
      clearInterval(this.slideLoopInterval);
    }
    if (this.idleMouseTimer > 0) {
      clearTimeout(this.idleMouseTimer);
    }
    window === null || window === void 0 ? void 0 : window.removeEventListener('resize', this.debounceWindowResize);
  }
  initRtl() {
    return new Promise((resolve) => {
      if (document && document.documentElement) {
        const htmlDir = document.documentElement.getAttribute('dir');
        this.rtl = htmlDir && htmlDir === 'rtl';
      }
      resolve();
    });
  }
  initWindowResize() {
    window === null || window === void 0 ? void 0 : window.addEventListener('resize', this.debounceWindowResize);
  }
  async initSlideSize() {
    if (!this.embedded) {
      await this.initSlideSizeStandard();
      return;
    }
    await this.initSlideSizeEmbedded();
  }
  async initSlideSizeStandard() {
    var _a, _b, _c, _d, _e;
    const sliderSize = this.containerSize();
    (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--slide-width', `${sliderSize.width}px`);
    if (this.dir === 'papyrus') {
      (_b = this.containerRef) === null || _b === void 0 ? void 0 : _b.style.setProperty('--slide-min-height', `${sliderSize.height}px`);
      (_c = this.containerRef) === null || _c === void 0 ? void 0 : _c.style.removeProperty('--slide-height');
    }
    else {
      (_d = this.containerRef) === null || _d === void 0 ? void 0 : _d.style.removeProperty('--slide-min-height');
      (_e = this.containerRef) === null || _e === void 0 ? void 0 : _e.style.setProperty('--slide-height', `${sliderSize.height}px`);
    }
    await this.initFontSize({ height: sliderSize.height, width: sliderSize.width });
  }
  containerSize() {
    return containerSize({ embedded: this.embedded, container: this.containerRef });
  }
  async initSlideSizeEmbedded() {
    if (!this.containerRef || !this.containerRef.offsetParent) {
      return;
    }
    const offsetParent = this.containerRef.offsetParent;
    if (offsetParent.clientWidth > 0) {
      this.containerRef.style.setProperty('--slide-width', '' + offsetParent.clientWidth + 'px');
    }
    if (offsetParent.clientHeight > 0 && this.dir === 'papyrus') {
      this.containerRef.style.setProperty('--slide-min-height', '' + offsetParent.clientHeight + 'px');
      this.containerRef.style.removeProperty('--slide-height');
    }
    if (offsetParent.clientHeight > 0 && this.dir !== 'papyrus') {
      this.containerRef.style.removeProperty('--slide-min-height');
      this.containerRef.style.setProperty('--slide-height', '' + offsetParent.clientHeight + 'px');
    }
    await this.initFontSize({ height: offsetParent.clientHeight, width: offsetParent.clientWidth });
  }
  async initFontSize({ height, width }) {
    var _a, _b;
    // 576px height = font-size 16px or 1em (relative to the font-size of its direct or nearest parent)
    const fontSize = height / 576;
    const ratioFontSize = ((width / 16) * 9) / 576;
    (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--slide-auto-font-size', `${fontSize}em`);
    (_b = this.containerRef) === null || _b === void 0 ? void 0 : _b.style.setProperty('--slide-auto-ratio-font-size', `${ratioFontSize}em`);
  }
  initKeyboardAssist() {
    if (document && this.keyboard) {
      document.addEventListener('keydown', this.keyboardAssist, { passive: true });
    }
  }
  async initLazyLoadContent() {
    if (window && 'IntersectionObserver' in window) {
      await this.deferIntersectionObserverLoad();
    }
    else {
      await this.lazyLoadAllContent();
    }
  }
  async deferIntersectionObserverLoad() {
    const slides = this.el.querySelectorAll('.deckgo-slide-container');
    if (!slides || slides.length <= 0) {
      return;
    }
    if (this.observer) {
      this.observer.disconnect();
    }
    this.observer = new IntersectionObserver(this.onIntersection, {
      rootMargin: '300px',
      threshold: 0
    });
    slides.forEach((slide) => {
      this.observer.observe(slide);
    });
  }
  async toggleKeyboardAssist(state) {
    if (!this.keyboard) {
      return;
    }
    if (state) {
      document.addEventListener('keydown', this.keyboardAssist, { passive: true });
    }
    else {
      // @ts-ignore
      document.removeEventListener('keydown', this.keyboardAssist, { passive: true });
    }
  }
  async onDirectionMobileChange() {
    await this.initDirection();
  }
  async onDirectionChange() {
    await this.initDirection();
  }
  async initDirection() {
    this.dir = this.mobile ? this.directionMobile : this.direction;
  }
  /* BEGIN: Handle swipe */
  mousedown($event) {
    this.start($event);
  }
  touchstart($event) {
    this.start($event);
  }
  async mouseup($event) {
    await this.stop($event);
  }
  async touchend($event) {
    await this.stop($event);
  }
  async mousemove($event) {
    await this.move($event);
  }
  async touchmove($event) {
    await this.move($event);
  }
  async dblclick() {
    this.resetStart();
  }
  async contextMenu() {
    this.resetStart();
  }
  scrolling($event) {
    this.block = $event ? $event.detail : false;
  }
  async keypress() {
    await this.clearMouseCursorTimer(true);
  }
  start($event) {
    const css = window.getComputedStyle($event.target);
    this.block =
      (css === null || css === void 0 ? void 0 : css.userSelect) === 'text' ||
        (css === null || css === void 0 ? void 0 : css.getPropertyValue('-webkit-user-select')) === 'text' ||
        (css === null || css === void 0 ? void 0 : css.getPropertyValue('-moz-user-select')) === 'text' ||
        (css === null || css === void 0 ? void 0 : css.getPropertyValue('-ms-user-select')) === 'text';
    this.startX = unifyEvent($event).clientX;
    this.startY = unifyEvent($event).clientY;
  }
  async move($event) {
    var _a;
    await this.clearMouseCursorTimer(true);
    if (this.block) {
      return;
    }
    if (this.animation !== 'slide') {
      return;
    }
    const delta = await this.getDelta($event);
    if (!delta) {
      return;
    }
    this.moveX(delta);
    this.moveY(delta);
    (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--transformXDuration', '0ms');
  }
  moveX(delta) {
    var _a;
    if (this.dir !== 'horizontal') {
      return;
    }
    const transformX = delta.swipeNext ? this.deckMove - delta.deltaX : this.deckMove + delta.deltaX;
    (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--transformX', transformX + 'px');
    this.slideDrag.emit(transformX);
  }
  moveY(delta) {
    var _a;
    if (this.dir !== 'vertical') {
      return;
    }
    const transformY = delta.swipeNext ? this.deckMove - delta.deltaY : this.deckMove + delta.deltaY;
    (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--transformY', transformY + 'px');
    this.slideDrag.emit(transformY);
  }
  async stop($event) {
    if (this.block) {
      return;
    }
    const delta = await this.getDelta($event);
    await this.swipeSlide(delta);
    this.resetStart();
  }
  swipeSlide(delta, emitEvent = true) {
    return new Promise(async (resolve) => {
      if (!delta || !window) {
        resolve();
        return;
      }
      let couldSwipeNext = delta.swipeNext && this.activeIndex < this.length - 1;
      let couldSwipePrevious = !delta.swipeNext && this.activeIndex > 0;
      if (this.rtl) {
        couldSwipeNext = delta.swipeNext && this.activeIndex > 0;
        couldSwipePrevious = !delta.swipeNext && this.activeIndex < this.length - 1;
      }
      if (couldSwipeNext || couldSwipePrevious) {
        const sliderSize = this.containerSize();
        const autoSwipeHorizontal = delta.deltaX > sliderSize.width / this.autoSwipeRatio;
        const autoSwipeVertical = delta.deltaY > sliderSize.height / this.autoSwipeRatio;
        if (autoSwipeHorizontal || autoSwipeVertical) {
          this.deckMove =
            this.dir !== 'horizontal'
              ? delta.swipeNext
                ? this.deckMove - sliderSize.height
                : this.deckMove + sliderSize.height
              : delta.swipeNext
                ? this.deckMove - sliderSize.width
                : this.deckMove + sliderSize.width;
          if (this.isNextChange(delta.swipeNext)) {
            this.activeIndex++;
            if (emitEvent) {
              this.slideNextDidChange.emit(this.activeIndex);
            }
          }
          else {
            this.activeIndex--;
            if (emitEvent) {
              this.slidePrevDidChange.emit(this.activeIndex);
            }
          }
        }
      }
      await this.doSwipeSlide();
      resolve();
    });
  }
  isNextChange(swipeLeft) {
    return (swipeLeft && !this.rtl) || (!swipeLeft && this.rtl);
  }
  doSwipeSlide(speed) {
    return new Promise((resolve) => {
      var _a, _b, _c, _d;
      if (this.dir === 'horizontal') {
        (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--transformX', this.deckMove + 'px');
      }
      if (this.dir === 'vertical') {
        (_b = this.containerRef) === null || _b === void 0 ? void 0 : _b.style.setProperty('--transformY', this.deckMove + 'px');
      }
      if (this.animation === 'slide') {
        (_c = this.containerRef) === null || _c === void 0 ? void 0 : _c.style.setProperty('--transformXDuration', '' + (!isNaN(speed) ? speed : 300) + 'ms');
      }
      else {
        (_d = this.containerRef) === null || _d === void 0 ? void 0 : _d.style.setProperty('--transformXDuration', '0ms');
      }
      if (this.dir === 'papyrus') {
        const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (this.activeIndex + 1) + ')');
        // HACK: Chrome does not perform any scroll if the behavior 'smooth' is set and the event is triggered programmatically or with the keyboard aka only works if triggered with a click event
        setTimeout(() => {
          slide === null || slide === void 0 ? void 0 : slide.scrollIntoView(this.animation === 'none' ? null : { behavior: 'smooth', block: 'nearest' });
        }, 0);
      }
      this.slideWillChange.emit(this.deckMove);
      this.resetStart();
      resolve();
    });
  }
  resetStart() {
    this.startX = null;
    this.startY = null;
  }
  async getDelta($event) {
    if (!this.startX && this.dir === 'horizontal') {
      return null;
    }
    if (!this.startY && this.dir === 'vertical') {
      return null;
    }
    if (this.dir === 'papyrus') {
      return null;
    }
    const currentX = unifyEvent($event).clientX;
    const currentY = unifyEvent($event).clientY;
    if (this.startX === currentX && this.dir === 'horizontal') {
      return null;
    }
    else if (this.startY === currentY && this.dir === 'vertical') {
      return null;
    }
    const swipeLeft = this.startX > currentX && this.dir === 'horizontal';
    const swipeTop = this.startY > currentY && this.dir === 'vertical';
    return {
      swipeNext: swipeLeft || swipeTop,
      deltaX: swipeLeft ? this.startX - currentX : currentX - this.startX,
      deltaY: swipeTop ? this.startY - currentY : currentY - this.startY
    };
  }
  /* END: Handle swipe */
  /* BEGIN: Slide length and active index */
  async slideDidLoad() {
    await this.updateLength();
    await this.afterSlidesDidLoad();
  }
  async updateLength() {
    const filteredSlides = await this.getDefinedFilteredSlides();
    this.length = filteredSlides ? filteredSlides.length : 0;
  }
  afterSlidesDidLoad() {
    return new Promise(async (resolve) => {
      const deckDefinition = await this.getDeckDefinition();
      if (deckDefinition && deckDefinition.slides && deckDefinition.slides.length > 0) {
        this.slidesDidLoad.emit(deckDefinition);
        await this.onAllSlidesDidLoad();
        this.deckDidLoad.emit();
      }
      resolve();
    });
  }
  onAllSlidesDidLoad() {
    return new Promise(async (resolve) => {
      const filteredSlides = await this.getDefinedFilteredSlides();
      const promises = [];
      promises.push(this.initLazyLoadContent());
      promises.push(DeckdeckgoDeckBackgroundUtils.loadSlots(this.el, filteredSlides, 'background', this.cloneBackground));
      if (this.dir !== 'papyrus') {
        promises.push(DeckdeckgoDeckBackgroundUtils.cloneSlots(this.el, filteredSlides, 'actions'));
        promises.push(DeckdeckgoDeckBackgroundUtils.loadSlots(this.el, filteredSlides, 'header'));
        promises.push(DeckdeckgoDeckBackgroundUtils.loadSlots(this.el, filteredSlides, 'footer'));
      }
      else if (filteredSlides && filteredSlides.length > 0) {
        promises.push(DeckdeckgoDeckBackgroundUtils.cloneSlot(this.el, filteredSlides[0], 'actions'));
        promises.push(DeckdeckgoDeckBackgroundUtils.loadSlot(this.el, filteredSlides[0], 'header'));
        promises.push(DeckdeckgoDeckBackgroundUtils.loadSlot(this.el, filteredSlides[filteredSlides.length - 1], 'footer'));
      }
      // In standard case, we want to be able to reveal elements or not, as we wish but if we set reveal to false, we want to display everything straight at the begin.
      // Or we display also all reveal elements on mobile devices as there is no keyboard on mobile device to reveal elements
      // Also, no reveal for papyrus as we can scroll
      if (!this.reveal || (!this.revealOnMobile && this.mobile) || this.dir === 'papyrus') {
        promises.push(this.revealAllContent());
      }
      await Promise.all(promises);
      resolve();
    });
  }
  getDeckDefinition() {
    return new Promise(async (resolve) => {
      const loadedSlides = this.el.querySelectorAll('.deckgo-slide-container.hydrated');
      if (!loadedSlides || loadedSlides.length <= 0) {
        resolve(null);
        return;
      }
      const filteredSlides = await this.getDefinedFilteredSlides();
      const definedSlidesLength = filteredSlides ? filteredSlides.length : 0;
      // Are all slides loaded?
      if (loadedSlides.length !== definedSlidesLength) {
        resolve(null);
        return;
      }
      const orderedSlidesTagNames = [];
      for (const slide of Array.from(loadedSlides)) {
        const attributes = await getAttributesDefinition(slide.attributes);
        orderedSlidesTagNames.push({
          template: slide.tagName ? slide.tagName.toLowerCase() : undefined,
          content: slide.innerHTML,
          attributes: attributes
        });
      }
      const attributes = await getAttributesDefinition(this.el.attributes);
      const background = this.el.querySelector(":scope > [slot='background']");
      const deck = {
        slides: orderedSlidesTagNames,
        attributes: attributes,
        background: background ? background.innerHTML : null,
        reveal: this.reveal,
        revealOnMobile: this.revealOnMobile
      };
      resolve(deck);
    });
  }
  getSlideDefinition(index) {
    return new Promise(async (resolve) => {
      const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (index + 1) + ')');
      const definition = await getSlideDefinition(slide);
      resolve(definition);
    });
  }
  getDefinedFilteredSlides() {
    return new Promise(async (resolve) => {
      const definedSlides = this.el.children;
      const filteredSlides = await this.filterSlides(definedSlides);
      resolve(filteredSlides);
    });
  }
  loadBackground() {
    return this.loadSlots('background', this.cloneBackground);
  }
  loadHeader() {
    return this.loadSlots('header');
  }
  loadFooter() {
    return this.loadSlots('footer');
  }
  async loadSlots(slotName, clone = true) {
    const filteredSlides = await this.getDefinedFilteredSlides();
    if (!filteredSlides || filteredSlides.length <= 0) {
      return;
    }
    await DeckdeckgoDeckBackgroundUtils.removeSlots(filteredSlides, slotName);
    await DeckdeckgoDeckBackgroundUtils.loadSlots(this.el, filteredSlides, slotName, clone);
  }
  // The last children might be slots (background, note or action)
  filterSlides(definedSlides) {
    return new Promise((resolve) => {
      if (!definedSlides || definedSlides.length <= 0) {
        resolve(null);
        return;
      }
      const slides = Array.from(definedSlides).filter((slide) => {
        return !slide.hasAttribute('slot') || slide.getAttribute('slot') === '';
      });
      resolve(slides);
    });
  }
  isBeginning() {
    return new Promise((resolve) => {
      resolve(this.activeIndex === 0);
    });
  }
  isEnd() {
    return new Promise((resolve) => {
      resolve(this.activeIndex === this.length - 1);
    });
  }
  getActiveIndex() {
    return new Promise((resolve) => {
      resolve(this.activeIndex);
    });
  }
  getLength() {
    return new Promise((resolve) => {
      resolve(this.length);
    });
  }
  /* END: Slide length and active index */
  /* BEGIN: Manual sliding */
  async slideNext(slideAnimation, emitEvent) {
    await this.slideNextPrev(!this.rtl, slideAnimation, emitEvent);
  }
  async slidePrev(slideAnimation, emitEvent) {
    await this.slideNextPrev(this.rtl, slideAnimation, emitEvent);
  }
  async slideNextPrev(swipeNext, slideAnimation = true, emitEvent) {
    const slider = this.el.shadowRoot.querySelector('div.deckgo-deck');
    if (!slider || !window) {
      return;
    }
    let couldSwipe;
    if (!slideAnimation) {
      couldSwipe = true;
    }
    else {
      couldSwipe = await this.beforeSwipe(this.isNextChange(swipeNext));
    }
    // We might want first to show hide stuffs in the slide before swiping
    if (couldSwipe) {
      const sliderSize = this.containerSize();
      const deltaX = {
        swipeNext,
        deltaX: sliderSize.width,
        deltaY: sliderSize.height
      };
      await this.swipeSlide(deltaX, emitEvent);
      await this.afterSwipe(swipeNext);
    }
    else if (emitEvent) {
      if (swipeNext) {
        this.slideNextDidAnimate.emit();
      }
      else {
        this.slidePrevDidAnimate.emit();
      }
    }
  }
  beforeSwipe(enter) {
    return new Promise(async (resolve) => {
      const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (this.activeIndex + 1) + ')');
      if (!slide) {
        // If we find no slide, we are cool something went wrong but the talk/show must go on
        resolve(true);
      }
      else {
        const result = await slide.beforeSwipe(enter, this.reveal);
        resolve(result);
      }
    });
  }
  afterSwipe(swipeLeft) {
    return new Promise(async (resolve) => {
      const indexPreviousSlide = swipeLeft ? this.activeIndex - 1 : this.activeIndex + 1;
      if (isNaN(indexPreviousSlide) || indexPreviousSlide < 0 || indexPreviousSlide > this.length) {
        resolve();
        return;
      }
      const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (indexPreviousSlide + 1) + ')');
      if (!slide) {
        // Might be a swipe after the first or last slide
        resolve();
      }
      else {
        await slide.afterSwipe();
        resolve();
      }
    });
  }
  async lazyLoadContent(index) {
    const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (index + 1) + ')');
    if (slide) {
      await slide.lazyLoadContent();
    }
  }
  async slideTo(index, speed, emitEvent = true) {
    if (index > this.length || index < 0) {
      return;
    }
    const slider = this.el.shadowRoot.querySelector('div.deckgo-deck');
    if (!slider || !window) {
      return;
    }
    const sliderSize = this.containerSize();
    this.deckMove = index * (this.dir === 'horizontal' ? sliderSize.width : sliderSize.height) * (this.rtl ? 1 : -1);
    this.activeIndex = index;
    await this.doSwipeSlide(speed);
    // In case we are sliding to a slide which received focus before being displayed (previously outside viewport)
    // a scroll on the parent element would have been applied by the browser. Therefore we set it back to the origin, as we takes care of the positioning of the slider.
    slider.parentElement.scrollTo(0, 0);
    if (emitEvent) {
      this.slideToChange.emit(index);
    }
  }
  async deleteActiveSlide(removeChild = true) {
    if (this.activeIndex > this.length || this.activeIndex < 0) {
      return;
    }
    if (removeChild) {
      const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (this.activeIndex + 1) + ')');
      slide === null || slide === void 0 ? void 0 : slide.parentElement.removeChild(slide);
    }
    this.activeIndex = this.activeIndex > 0 ? this.activeIndex - 1 : 0;
    this.length = this.length > 0 ? this.length - 1 : 0;
    if (this.length > 0) {
      await this.slideTo(this.activeIndex, 0);
    }
  }
  async blockSlide(block) {
    this.block = block;
    // If we want to block, then we reset then previous start position as we don't want to start the slide to scroll when the blocking will be resolved
    if (this.block) {
      this.resetStart();
    }
  }
  /* END: Manual sliding */
  /* BEGIN: Full screen */
  toggleFullScreen() {
    return new Promise(async (resolve) => {
      const doc = window.document;
      const docEl = doc.documentElement;
      // @ts-ignore
      // prettier-ignore
      const requestFullScreen = docEl.requestFullscreen || docEl.mozRequestFullScreen || docEl.webkitRequestFullScreen || docEl.msRequestFullscreen;
      // @ts-ignore
      const cancelFullScreen = doc.exitFullscreen || doc.mozCancelFullScreen || doc.webkitExitFullscreen || doc.msExitFullscreen;
      // @ts-ignore
      if (!doc.fullscreenElement && !doc.mozFullScreenElement && !doc.webkitFullscreenElement && !doc.msFullscreenElement) {
        requestFullScreen.call(docEl);
      }
      else {
        cancelFullScreen.call(doc);
      }
      resolve();
    });
  }
  async hideOrClearMouseCursorTimer(toggleFullscreen) {
    if (toggleFullscreen) {
      this.fullscreen = true;
      // We are entering fullscreen mode now, therefore we don't want to want two seconds to hide the mouse
      setTimeout(async () => {
        await this.showHideMouseCursor(false);
      }, 500);
      this.hideMouseCursorWithDelay();
    }
    else {
      await this.clearMouseCursorTimer(false);
      this.fullscreen = false;
    }
  }
  async clearMouseCursorTimer(hideWithDelay) {
    if (!this.fullscreen) {
      return;
    }
    if (this.idleMouseTimer > 0) {
      clearTimeout(this.idleMouseTimer);
    }
    await this.showHideMouseCursor(true);
    if (hideWithDelay) {
      this.hideMouseCursorWithDelay();
    }
  }
  hideMouseCursorWithDelay() {
    if (!this.fullscreen) {
      return;
    }
    this.idleMouseTimer = window.setTimeout(async () => {
      await this.showHideMouseCursor(false);
    }, this.idleMouseTimeout);
  }
  showHideMouseCursor(show) {
    return new Promise((resolve) => {
      var _a;
      if (!this.fullscreen) {
        resolve();
        return;
      }
      if (!this.cursorHidden && show) {
        // Cursor already displayed, we don't want to touch the style multiple times if not needed
        resolve();
        return;
      }
      (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('cursor', show ? 'inherit' : 'none');
      this.mouseInactivity.emit(show);
      this.cursorHidden = !show;
      resolve();
    });
  }
  async showHideActionsSlot(toggleFullscreen) {
    var _a, _b;
    if (toggleFullscreen) {
      (_a = this.containerRef) === null || _a === void 0 ? void 0 : _a.style.setProperty('--slide-actions-display', 'none');
    }
    else {
      (_b = this.containerRef) === null || _b === void 0 ? void 0 : _b.style.removeProperty('--slide-actions-display');
    }
  }
  /* END: Full screen */
  /* BEGIN: Utils */
  async doPrint() {
    const dir = this.direction;
    window.addEventListener('afterprint', () => (this.direction = dir), { once: true });
    const onRender = async (_mutations, observer) => {
      observer.disconnect();
      await Promise.all([this.lazyLoadAllContent(), this.revealAllContent()]);
      window.print();
    };
    const docObserver = new MutationObserver(onRender);
    docObserver.observe(this.el, { attributes: true });
    this.direction = 'vertical';
  }
  lazyLoadAllContent() {
    const promises = [];
    for (let i = 0; i < this.length; i++) {
      promises.push(this.lazyLoadContent(i));
    }
    return Promise.all(promises);
  }
  /* END: Utils */
  /* BEGIN: Reveal */
  async onRevealChange() {
    if (!this.reveal) {
      await this.revealAllContent();
    }
    else {
      await this.redoRevealContent();
    }
  }
  revealAllContent() {
    return new Promise(async (resolve) => {
      const promises = [];
      for (let i = 0; i < this.length; i++) {
        promises.push(this.revealContent(i));
      }
      await Promise.all(promises);
      resolve();
    });
  }
  redoRevealContent() {
    return new Promise(async (resolve) => {
      // If we switch back to standard mode, reveal previous slide and hide the "reveal" content of this and next slides
      const promises = [];
      for (let i = 0; i < this.length; i++) {
        if (i < this.activeIndex) {
          promises.push(this.revealContent(i));
        }
        else {
          promises.push(this.hideContent(i));
        }
      }
      await Promise.all(promises);
      resolve();
    });
  }
  async revealContent(index) {
    const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (index + 1) + ')');
    if (slide) {
      await slide.revealContent();
    }
  }
  async hideContent(index) {
    const slide = this.el.querySelector('.deckgo-slide-container:nth-child(' + (index + 1) + ')');
    if (slide) {
      await slide.hideContent();
    }
  }
  /* END: Reveal */
  /* BEGIN: AutoSlide */
  async initAutoSlide() {
    if (this.autoSlide === 'true') {
      await this.onAutoSlide();
    }
  }
  async onAutoSlide() {
    let idleMouseTimer;
    this.idleSlideLoopTimer = window.setTimeout(() => (idleMouseTimer = this.idleMouseTimer), this.idleMouseTimeout);
    if (this.autoSlide === 'true') {
      this.slideLoopInterval = window.setInterval(async () => {
        if (idleMouseTimer === this.idleMouseTimer) {
          const end = await this.isEnd();
          if (end) {
            await this.slideTo(0, 0);
          }
          else {
            await this.slideNext(true);
          }
        }
        else {
          idleMouseTimer = this.idleMouseTimer;
        }
      }, this.autoSlideInterval);
    }
    else {
      clearInterval(this.slideLoopInterval);
    }
  }
  /* END: AutoSlide */
  render() {
    return (h(Host, { class: `${this.dir}` },
      h("main", null,
        this.renderAnimation(),
        h("div", { class: "deckgo-deck", ref: (el) => (this.containerRef = el) },
          h("slot", null),
          h("slot", { name: "actions" }),
          h("slot", { name: "background" })),
        h("slot", { name: "pager" }))));
  }
  renderAnimation() {
    if (this.animation !== 'fade' || this.dir === 'papyrus') {
      return h(AnimationSlide, null);
    }
    return [h(HideSlides, null), h(RevealSlide, { index: this.activeIndex })];
  }
  static get is() { return "deckgo-deck"; }
  static get encapsulation() { return "shadow"; }
  static get originalStyleUrls() { return {
    "$": ["deck.scss"]
  }; }
  static get styleUrls() { return {
    "$": ["deck.css"]
  }; }
  static get properties() { return {
    "keyboard": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "keyboard",
      "reflect": false,
      "defaultValue": "true"
    },
    "embedded": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "embedded",
      "reflect": false,
      "defaultValue": "false"
    },
    "cloneBackground": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "clone-background",
      "reflect": false,
      "defaultValue": "true"
    },
    "reveal": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "reveal",
      "reflect": false,
      "defaultValue": "true"
    },
    "revealOnMobile": {
      "type": "boolean",
      "mutable": false,
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "reveal-on-mobile",
      "reflect": false,
      "defaultValue": "false"
    },
    "animation": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'slide' | 'fade' | 'none'",
        "resolved": "\"fade\" | \"none\" | \"slide\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "animation",
      "reflect": true,
      "defaultValue": "'slide'"
    },
    "direction": {
      "type": "string",
      "mutable": true,
      "complexType": {
        "original": "'horizontal' | 'vertical' | 'papyrus'",
        "resolved": "\"horizontal\" | \"papyrus\" | \"vertical\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "direction",
      "reflect": true,
      "defaultValue": "'horizontal'"
    },
    "directionMobile": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'horizontal' | 'vertical' | 'papyrus'",
        "resolved": "\"horizontal\" | \"papyrus\" | \"vertical\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "direction-mobile",
      "reflect": true,
      "defaultValue": "'papyrus'"
    },
    "autoSlide": {
      "type": "string",
      "mutable": false,
      "complexType": {
        "original": "'true' | 'false'",
        "resolved": "\"false\" | \"true\"",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "auto-slide",
      "reflect": false,
      "defaultValue": "'false'"
    },
    "autoSlideInterval": {
      "type": "number",
      "mutable": false,
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      },
      "required": false,
      "optional": false,
      "docs": {
        "tags": [],
        "text": ""
      },
      "attribute": "auto-slide-interval",
      "reflect": false,
      "defaultValue": "5000"
    }
  }; }
  static get states() { return {
    "rtl": {},
    "activeIndex": {},
    "dir": {}
  }; }
  static get events() { return [{
      "method": "slidesDidLoad",
      "name": "slidesDidLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "any",
        "resolved": "any",
        "references": {}
      }
    }, {
      "method": "slideNextDidAnimate",
      "name": "slideNextDidAnimate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "slidePrevDidAnimate",
      "name": "slidePrevDidAnimate",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "slideNextDidChange",
      "name": "slideNextDidChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "slidePrevDidChange",
      "name": "slidePrevDidChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "slideToChange",
      "name": "slideToChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "slideDrag",
      "name": "slideDrag",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "slideWillChange",
      "name": "slideWillChange",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "number",
        "resolved": "number",
        "references": {}
      }
    }, {
      "method": "deckDidLoad",
      "name": "deckDidLoad",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "void",
        "resolved": "void",
        "references": {}
      }
    }, {
      "method": "mouseInactivity",
      "name": "mouseInactivity",
      "bubbles": true,
      "cancelable": true,
      "composed": true,
      "docs": {
        "tags": [],
        "text": ""
      },
      "complexType": {
        "original": "boolean",
        "resolved": "boolean",
        "references": {}
      }
    }]; }
  static get methods() { return {
    "initSlideSize": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "toggleKeyboardAssist": {
      "complexType": {
        "signature": "(state: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getDeckDefinition": {
      "complexType": {
        "signature": "() => Promise<DeckdeckgoDeckDefinition | null>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "DeckdeckgoDeckDefinition": {
            "location": "import",
            "path": "@deckdeckgo/types"
          },
          "NodeListOf": {
            "location": "global"
          },
          "HTMLElement": {
            "location": "global"
          },
          "DeckdeckgoSlideDefinition": {
            "location": "import",
            "path": "@deckdeckgo/types"
          },
          "DeckdeckgoAttributeDefinition": {
            "location": "import",
            "path": "@deckdeckgo/types"
          }
        },
        "return": "Promise<DeckdeckgoDeckDefinition>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getSlideDefinition": {
      "complexType": {
        "signature": "(index: number) => Promise<DeckdeckgoSlideDefinition | null>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "DeckdeckgoSlideDefinition": {
            "location": "import",
            "path": "@deckdeckgo/types"
          },
          "HTMLElement": {
            "location": "global"
          }
        },
        "return": "Promise<DeckdeckgoSlideDefinition>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "loadBackground": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "loadHeader": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "loadFooter": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "isBeginning": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "isEnd": {
      "complexType": {
        "signature": "() => Promise<boolean>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<boolean>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getActiveIndex": {
      "complexType": {
        "signature": "() => Promise<number>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<number>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "getLength": {
      "complexType": {
        "signature": "() => Promise<number>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<number>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "slideNext": {
      "complexType": {
        "signature": "(slideAnimation?: boolean, emitEvent?: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "slidePrev": {
      "complexType": {
        "signature": "(slideAnimation?: boolean, emitEvent?: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "slideTo": {
      "complexType": {
        "signature": "(index: number, speed?: number | undefined, emitEvent?: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }, {
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLElement": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "deleteActiveSlide": {
      "complexType": {
        "signature": "(removeChild?: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          },
          "HTMLElement": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "blockSlide": {
      "complexType": {
        "signature": "(block: boolean) => Promise<void>",
        "parameters": [{
            "tags": [],
            "text": ""
          }],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "toggleFullScreen": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "doPrint": {
      "complexType": {
        "signature": "() => Promise<void>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          },
          "MutationRecord": {
            "location": "global"
          },
          "MutationObserver": {
            "location": "global"
          }
        },
        "return": "Promise<void>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    },
    "lazyLoadAllContent": {
      "complexType": {
        "signature": "() => Promise<void[]>",
        "parameters": [],
        "references": {
          "Promise": {
            "location": "global"
          }
        },
        "return": "Promise<void[]>"
      },
      "docs": {
        "text": "",
        "tags": []
      }
    }
  }; }
  static get elementRef() { return "el"; }
  static get watchers() { return [{
      "propName": "directionMobile",
      "methodName": "onDirectionMobileChange"
    }, {
      "propName": "direction",
      "methodName": "onDirectionChange"
    }, {
      "propName": "reveal",
      "methodName": "onRevealChange"
    }, {
      "propName": "autoSlide",
      "methodName": "onAutoSlide"
    }]; }
  static get listeners() { return [{
      "name": "mousedown",
      "method": "mousedown",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "touchstart",
      "method": "touchstart",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "mouseup",
      "method": "mouseup",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "touchend",
      "method": "touchend",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "mousemove",
      "method": "mousemove",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "touchmove",
      "method": "touchmove",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "dblclick",
      "method": "dblclick",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "contextmenu",
      "method": "contextMenu",
      "target": undefined,
      "capture": false,
      "passive": true
    }, {
      "name": "scrolling",
      "method": "scrolling",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "keypress",
      "method": "keypress",
      "target": undefined,
      "capture": false,
      "passive": false
    }, {
      "name": "slideDidLoad",
      "method": "slideDidLoad",
      "target": undefined,
      "capture": false,
      "passive": false
    }]; }
}
